#!/usr/local/bin/python3
import sys
from xml.etree.ElementTree import VERSION
from shapely import geometry
import math
import math as mt

a = 6378137.0  
b = 6356752.31414 
a = 6378137.0 
f = 1 / 298.257223563 
b = 6356752.31414  
sqrt = mt.sqrt
sin = mt.sin
cos = mt.cos
atan = mt.atan
atan2 = mt.atan2
fabs = math.fabs

def rad2angle(r):
    a = r * 180.0 / mt.pi
    return a

def angle2rad(a):
    r = a * mt.pi / 180.0
    return r

def XYZ2BLH(X, Y, Z):
    R = sqrt(X * X + Y * Y)
    B0 = atan2(Z, R)
    while 1:
        N = a / sqrt(1.0 - f * (2 - f) * sin(B0) * sin(B0))
        B_ = atan2(Z + N * f * (2 - f) * sin(B0), R)
        if (fabs(B_ - B0) < 1.0e-7):
            break
        B0 = B_
    L_ = atan2(Y, X)
    H = R / cos(B_) - N
    B = rad2angle(B_)
    L = rad2angle(L_)
    return B, L, H

def BLH2XYZ(B, L, H):
    B = angle2rad(B)  
    L = angle2rad(L)  
    e = sqrt((a ** 2 - b ** 2) / (a ** 2)) 
    N = a / sqrt(1 - e * e * sin(B) * sin(B)) 
    X = (N + H) * cos(B) * cos(L)
    Y = (N + H) * cos(B) * sin(L)
    Z = (N * (1 - e * e) + H) * sin(B)
    return X, Y, Z 

def readrequest(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        VRS=[]
        num=0
        for line in readfile:
            if num>1:
                vrs=[]
                data_line=line.strip('\n').split()
                XYZ=BLH2XYZ(float(data_line[1]),float(data_line[0]),float(data_line[2]))
                vrs.append(data_line[15])#name
                for i in range(len(XYZ)):vrs.append(XYZ[i])  
                VRS.append(vrs)
            num=num+1
        return VRS

def readstationXYZ(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        Station=[]
        for line in readfile:
            station=[]
            data_line=line.strip('\n').split()
            station.append(data_line[0])#name
            station.append(float(data_line[1]))#X
            station.append(float(data_line[2]))#Y
            station.append(float(data_line[3]))#Z
            Station.append(station)
        return Station

def readtriangles(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        Triangles=[]
        num=0
        for line in readfile:
            triangle=[]
            data_line=line.strip('\n').split()
            if num>0:
                triangle.append(data_line[1])
                triangle.append(data_line[2])
                triangle.append(data_line[3])
                Triangles.append(triangle)
            num=num+1
        return Triangles
def find_Point(Station_XYZ,Pointname):
    for i in range(len(Station_XYZ)):
        if Pointname==Station_XYZ[i][0]:return (Station_XYZ[i][1],Station_XYZ[i][2],Station_XYZ[i][3])
             
def find_XYZ(Station_XYZ,Triangle):
    Points=[]
    for i in range(len(Triangle)):
        point=find_Point(Station_XYZ,Triangle[i])
        Points.append(point)
    return Points
def calculate_cos(point1,point2,point3):
    vector1=[point2[0]-point1[0],point2[1]-point1[1],point2[2]-point1[2]]
    vector2=[point3[0]-point1[0],point3[1]-point1[1],point3[2]-point1[2]]
    vec1vec2=vector1[0]*vector2[0]+vector1[1]*vector2[1]+vector1[2]*vector2[2]
    vec1=sqrt(vector1[0]*vector1[0]+vector1[1]*vector1[1]+vector1[2]*vector1[2])#向量模长
    vec2=sqrt(vector2[0]*vector2[0]+vector2[1]*vector2[1]+vector2[2]*vector2[2])
    return vec1vec2/(vec1*vec2)
    
def calculate_cos2judge(Points,judge1,judge2):
    Cos_triangle=[]
    for i in range(len(Points)):  
        if i==0:j=1;k=2
        if i==1:j=0;k=2
        if i==2:j=0;k=1
        Cos_triangle.append(calculate_cos(Points[i],Points[j],Points[k]))
    #根据方向余弦判断三角形构型
    #print(Cos_triangle)
    for i in range(len(Cos_triangle)):
        if Cos_triangle[i]<judge1:return 0
        if Cos_triangle[i]>judge2:return 0
    return 1
def find_minstation(points,VRS_point):
    min_dis=99999999.0
    min_id=3
    for i in range(len(points)):
        dx=points[i][0]-VRS_point[1]
        dy=points[i][1]-VRS_point[2]
        dz=points[i][2]-VRS_point[3]
        dis=sqrt(dx*dx+dy*dy+dz*dz)
        if dis<min_dis:min_dis=dis;min_id=i
    return min_id
def find_maxstation(points,VRS_point):
    max_dis=0.0
    max_id=3
    for i in range(len(points)):
        dx=points[i][0]-VRS_point[1]
        dy=points[i][1]-VRS_point[2]
        dz=points[i][2]-VRS_point[3]
        dis=sqrt(dx*dx+dy*dy+dz*dz)
        if dis>max_dis:max_dis=dis;max_id=i
    return max_id    
      
        
    
                       
# t_point = geometry.Point(t_x, t_y)
# temp_polygon = geometry.Polygon(points)#points：是由多个元组点组成的列表[(x1,y1),(x2,y2)...]
# if temp_polygon.covers(t_point):

if __name__  == '__main__':  
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    VRS_request=readrequest(sys.argv[1])
    #print(VRS_request)
    #print(cos(angle2rad(30)))#识别弧度
    Station_XYZ=readstationXYZ(sys.argv[2])
    Triangles=readtriangles(sys.argv[3])
    #print(VRS_request)
    #print(Station_XYZ)
    #print(Triangles)
    #先按照一个格网点选择的三角形进行判断
    t_point=geometry.Point(VRS_request[0][1], VRS_request[0][2])
    for i in range(len(Triangles)):
        points=find_XYZ(Station_XYZ,Triangles[i])#返回对应三角形的坐标信息（元组列表）
        Points=[]#存储XY
        for j in range(len(points)):
            p=(points[j][0],points[j][1])
            Points.append(p)
        temp_polygon = geometry.Polygon(Points)
        #print(points)
        #print(points[0][0])
        if temp_polygon.covers(t_point):
            #判断三角形的三个内角情况
            with open(sys.argv[10],'w',encoding='utf-8') as writefile:
                    writefile.write(Triangles[i][0]+" "+Triangles[i][1]+" "+Triangles[i][2]+"\n")
            #最大角阈值105度：-0.25881904510252085   最小角阈值35度：0.8191520442889918  30度：0.8660254037844387
            # print(points)#XYZ
            # print(Points)#XY
            # print(VRS_request[0])
            #print(cos(angle2rad(35)))#识别弧度
            judge1=cos(angle2rad(float(sys.argv[6])))
            judge2=cos(angle2rad(float(sys.argv[7])))
            judge=calculate_cos2judge(points,judge1,judge2)
            #print(judge)
            if judge==0:
                #对于三角形不满足构型时
                # print(VRS_request[0])
                # print(Triangles[i])#只有点名（对应坐标在points中）
                min_id=find_minstation(points,VRS_request[0])
                #print(min_id)
                max_id=find_maxstation(points,VRS_request[0])
                mid_id=3
                for k in range(len(points)):
                    if k !=min_id and k !=max_id:mid_id=k
                if int(sys.argv[9])<=1:
                    #print(1)
                    with open(sys.argv[8],'w',encoding='utf-8') as writefile:
                        writefile.write(Triangles[i][mid_id]+"\n")
                if int(sys.argv[9])>1:
                    #print(0)
                    with open(sys.argv[8],'w',encoding='utf-8') as writefile:
                        writefile.write(Triangles[i][min_id]+"\n")
                break
            if judge==1:
                #写出对应三角形的基线信息
                with open(sys.argv[4],'w',encoding='utf-8') as writefile:
                    writefile.write("nbaseline 3"+"\n")
                    str1="RECORD1:"+" "+Triangles[i][0]+" "+Triangles[i][1]+"\n"
                    str2="RECORD2:"+" "+Triangles[i][1]+" "+Triangles[i][2]+"\n"
                    str3="RECORD3:"+" "+Triangles[i][2]+" "+Triangles[i][0]+"\n"
                    writefile.write(str1)
                    writefile.write(str2)
                    writefile.write(str3)
                #print("Find a Triangle for VRS grid!")
                with open(sys.argv[5],'w',encoding='utf-8') as writefile:
                    writefile.write(Triangles[i][0]+" "+Triangles[i][1]+" "+Triangles[i][2]+"\n")
                break
        if i==(int(len(Triangles))-1):print("not find triangle in py_find_triangle.py!")
        
    