#!/usr/local/bin/python3
import sys
from xml.etree.ElementTree import VERSION
from shapely import geometry
import math
import math as mt

a = 6378137.0  
b = 6356752.31414 
a = 6378137.0 
f = 1 / 298.257223563 
b = 6356752.31414  
sqrt = mt.sqrt
sin = mt.sin
cos = mt.cos
atan = mt.atan
atan2 = mt.atan2
fabs = math.fabs

def rad2angle(r):
    a = r * 180.0 / mt.pi
    return a

def angle2rad(a):
    r = a * mt.pi / 180.0
    return r

def XYZ2BLH(X, Y, Z):
    R = sqrt(X * X + Y * Y)
    B0 = atan2(Z, R)
    while 1:
        N = a / sqrt(1.0 - f * (2 - f) * sin(B0) * sin(B0))
        B_ = atan2(Z + N * f * (2 - f) * sin(B0), R)
        if (fabs(B_ - B0) < 1.0e-7):
            break
        B0 = B_
    L_ = atan2(Y, X)
    H = R / cos(B_) - N
    B = rad2angle(B_)
    L = rad2angle(L_)
    return B, L, H

def BLH2XYZ(B, L, H):
    B = angle2rad(B)  
    L = angle2rad(L)  
    e = sqrt((a ** 2 - b ** 2) / (a ** 2)) 
    N = a / sqrt(1 - e * e * sin(B) * sin(B)) 
    X = (N + H) * cos(B) * cos(L)
    Y = (N + H) * cos(B) * sin(L)
    Z = (N * (1 - e * e) + H) * sin(B)
    return X, Y, Z 

def readstationXYZ(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        Station=[]
        for line in readfile:
            station=[]
            data_line=line.strip('\n').split()
            station.append(data_line[0])#name
            station.append(float(data_line[1]))#X
            station.append(float(data_line[2]))#Y
            station.append(float(data_line[3]))#Z
            Station.append(station)
        return Station

def readtriangles(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        Triangles=[]
        for line in readfile:
            triangle=[]
            data_line=line.strip('\n').split()
            triangle.append(data_line[0])
            triangle.append(data_line[1])
            triangle.append(data_line[2])
            Triangles.append(triangle)
        return Triangles
def find_Point(Station_XYZ,Pointname):
    for i in range(len(Station_XYZ)):
        if Pointname==Station_XYZ[i][0]:return (Station_XYZ[i][1],Station_XYZ[i][2],Station_XYZ[i][3])
             
def find_XYZ(Station_XYZ,Triangle):
    Points=[]
    for i in range(len(Triangle)):
        point=find_Point(Station_XYZ,Triangle[i])
        Points.append(point)
    return Points
#计算一个夹角的方向余弦
def calculate_cos(point1,point2,point3):
    vector1=[point2[0]-point1[0],point2[1]-point1[1],point2[2]-point1[2]]
    vector2=[point3[0]-point1[0],point3[1]-point1[1],point3[2]-point1[2]]
    vec1vec2=vector1[0]*vector2[0]+vector1[1]*vector2[1]+vector1[2]*vector2[2]
    vec1=sqrt(vector1[0]*vector1[0]+vector1[1]*vector1[1]+vector1[2]*vector1[2])#向量模长
    vec2=sqrt(vector2[0]*vector2[0]+vector2[1]*vector2[1]+vector2[2]*vector2[2])
    return vec1vec2/(vec1*vec2)

#计算一个三角形的方向余弦   
def calculate_cos2judge(Points):
    Cos_triangle=[]
    for i in range(len(Points)):  
        if i==0:j=1;k=2
        if i==1:j=0;k=2
        if i==2:j=0;k=1
        Cos_triangle.append(calculate_cos(Points[i],Points[j],Points[k]))
    return Cos_triangle

def find_max(Cos_triangle):
    max=0
    max_id=3
    for i in range(len(Cos_triangle)):
        if Cos_triangle[i]>max:max=Cos_triangle[i];max_id=i 
    return max_id
def find_min(all_triangle):
    min=1
    min_id=3
    for i in range(len(all_triangle)):
        if all_triangle[i]<min:min=all_triangle[i];min_id=i 
    return min_id
                       
# t_point = geometry.Point(t_x, t_y)
# temp_polygon = geometry.Polygon(points)#points：是由多个元组点组成的列表[(x1,y1),(x2,y2)...]
# if temp_polygon.covers(t_point):

if __name__  == '__main__':  
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    Station_XYZ=readstationXYZ(sys.argv[1])
    #print(Station_XYZ)
    Triangles=readtriangles(sys.argv[2])
    #print(Triangles)
    ALL_Cos_triangle=[]
    for i in range(len(Triangles)):
        points=find_XYZ(Station_XYZ,Triangles[i])#找出当前三角形三个点
        #print(points)
        ALL_Cos_triangle.append(calculate_cos2judge(points))
    #print(ALL_Cos_triangle)
    #寻找每个方向余弦的最大值，也就是最小角
    max_cos=[]
    for i in range(len(ALL_Cos_triangle)):
        max_id=find_max(ALL_Cos_triangle[i])
        max_cos.append(ALL_Cos_triangle[i][max_id])#选最小角
    #print(max_cos)
    triangle_id=find_min(max_cos)
    #print(Triangles[triangle_id])
    #写出对应三角形的基线信息
    with open(sys.argv[3],'w',encoding='utf-8') as writefile:
        writefile.write("nbaseline 3"+"\n")
        str1="RECORD1:"+" "+Triangles[triangle_id][0]+" "+Triangles[triangle_id][1]+"\n"
        str2="RECORD2:"+" "+Triangles[triangle_id][1]+" "+Triangles[triangle_id][2]+"\n"
        str3="RECORD3:"+" "+Triangles[triangle_id][2]+" "+Triangles[triangle_id][0]+"\n"
        writefile.write(str1)
        writefile.write(str2)
        writefile.write(str3)
    #print("Find a Triangle for VRS grid!")
    with open(sys.argv[4],'w',encoding='utf-8') as writefile:
        writefile.write(Triangles[triangle_id][0]+" "+Triangles[triangle_id][1]+" "+Triangles[triangle_id][2]+"\n")    
                
        
    